import java.time.format.*
import java.time.*

@Suppress("DSL_SCOPE_VIOLATION") // TODO: Remove once KTIJ-19369 is fixed
plugins {
    alias(libs.plugins.com.android.application)
    alias(libs.plugins.org.jetbrains.kotlin.android)
    kotlin("kapt")
}

android {
    namespace = "com.anjun.supervision"
    compileSdk = libs.versions.compileSdk.get().toInt()

    defaultConfig {
        applicationId = "com.anjun.supervision"
        minSdk = libs.versions.minSdk.get().toInt()
        targetSdk = libs.versions.targetSdk.get().toInt()
        versionCode = 1
        versionName = "1.0"

        testInstrumentationRunner = "androidx.test.runner.AndroidJUnitRunner"
        vectorDrawables {
            useSupportLibrary = true
        }
    }

    signingConfigs {
        create("release"){
            storeFile = file("supervisionWarehouseKey.jks")
            storePassword = "anjun2024"
            keyAlias = "supervisionApp"
            keyPassword = "anjun2024"
        }

        create("debugSigning"){
            storeFile = file("supervisionWarehouseKey.jks")
            storePassword = "anjun2024"
            keyAlias = "supervisionApp"
            keyPassword = "anjun2024"
        }
    }

    buildTypes {
        release {
            isMinifyEnabled = false
            proguardFiles(
                getDefaultProguardFile("proguard-android-optimize.txt"),
                "proguard-rules.pro"
            )
            signingConfig = signingConfigs.getByName("release")
        }

        debug {
            signingConfig = signingConfigs.getByName("debugSigning")
        }
    }

    android.applicationVariants.all {
        outputs.all {
            if (this is com.android.build.gradle.internal.api.ApkVariantOutputImpl) {
                val config = project.android.defaultConfig
                val versionName = config.versionName
                val formatter = DateTimeFormatter.ofPattern("yyyyMMddHHmm")
                val createTime = LocalDateTime.now().format(formatter)
                this.outputFileName = "AJSupervision_${this.name}_${versionName}_$createTime.apk"
            }
        }

    }

    compileOptions {
        sourceCompatibility = JavaVersion.VERSION_1_8
        targetCompatibility = JavaVersion.VERSION_1_8
    }

    kotlinOptions{
        jvmTarget = "1.8"
    }

    packaging {
        resources {
            excludes += "/META-INF/{AL2.0,LGPL2.1}"
        }
    }

    dataBinding{
        enable = true
    }
}

tasks.withType(org.jetbrains.kotlin.gradle.tasks.KotlinCompile::class.java){
    kotlinOptions {
        jvmTarget = "1.8"
    }
}


dependencies {

    implementation(libs.bundles.androidx)
    implementation(libs.cardview)
    implementation(libs.recyclerview)
    implementation(libs.constraintlayout)
    implementation(libs.flexbox)
    implementation(libs.bundles.okhttp)
    implementation(libs.bundles.retrofit)
    implementation(libs.mmkv)
    implementation(libs.kotlin.reflect)
    implementation(libs.paging)

    implementation(project(":core:core_tool"))
    implementation(project(":core:core_router"))
    implementation(project(":core:core_mvvm"))
    implementation(project(":core:network"))
    implementation(project(":features:feature_common:common_view"))
    implementation(project(":features:feature_common:common_base"))
    implementation(project(":data:data_main"))

    //需要初始化各feature
    implementation(project(":features:feature_main"))
    implementation(project(":service:service_main"))
}