package com.anjun.core.mvvm

import android.app.Activity
import java.lang.ref.WeakReference
import java.util.Stack

/**
 *
 * @author melle
 * @since 2024/03/28 11:11:10
 */
object ActivityManager {
    private val mStack = Stack<WeakReference<Activity>>()

    fun getCount() = mStack.size

    fun addActivity(activity: Activity) = mStack.add(WeakReference(activity))

    fun topActivity() = if (mStack.isEmpty()) null else mStack.peek().get()

    inline fun <reified T : Activity> findActivity(): T? =
        findActivity {
            it::class == T::class
        }?.get() as? T

    fun removeActivity(activity: Activity) = findActivity {
        it == activity
    }?.let { mStack.remove(it) }

    fun finishActivity(activity: Activity) {
        if (mStack.isEmpty()) {
            activity.finish()
        } else {
            findActivity {
                it == activity
            }?.let {
                it.get()?.finish()
                mStack.remove(it)
            }
        }
    }

    fun finishAllActivity() {
        if (mStack.isEmpty()) return
        mStack.forEach {
            it.get()?.finish()
        }
        mStack.clear()
    }

    fun findActivity(filter: (Activity) -> Boolean): WeakReference<Activity>? {
        return mStack.firstOrNull { reference ->
            reference.get()?.let {
                return@firstOrNull filter(it)
            } ?: mStack.remove(reference)
            return@firstOrNull false
        }
    }
}