package com.anjun.core.mvvm

import android.os.Bundle
import androidx.activity.OnBackPressedCallback
import androidx.appcompat.app.AppCompatActivity

/**
 *
 * @Description
 * @Author melle
 * @CreateTime 2024/03/08 15:11:10
 */
abstract class BaseActivity : AppCompatActivity() {

    private val mLayoutId: Int by lazy {
        getLayoutId()
    }

    protected open var backPressedCallbackEnable = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        onBackPressedDispatcher.addCallback(this,
            object : OnBackPressedCallback(backPressedCallbackEnable) {
                override fun handleOnBackPressed() {
                    onBackPressedCallback()
                }
            })
        ActivityManager.addActivity(this)
        setupContentView()
        savedInstanceState?.let { loadSavedState(it) }
        setupStatusBar()
        setup()
    }

    protected open fun setupContentView() {
        if (mLayoutId != 0) {
            setContentView(mLayoutId)
        }
    }

    protected open fun loadSavedState(savedInstanceState: Bundle) {}

    protected open fun setupStatusBar() {}

    protected open fun setup() {}

    abstract fun getLayoutId(): Int

    protected open fun onBackPressedCallback() {}

    override fun onDestroy() {
        super.onDestroy()
        ActivityManager.removeActivity(this)
    }
}