package com.anjun.core.mvvm.ext

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.viewbinding.ViewBinding
import java.lang.RuntimeException
import java.lang.reflect.ParameterizedType

/**
 *
 * @Description
 * @Author melle
 * @CreateTime 2024/03/08 15:58:32
 */
fun <VB : ViewBinding> Any.getViewBinding(
    inflater: LayoutInflater,
    position: Int = 0,
    parent: ViewGroup? = null,
    attachToRoot: Boolean = false
): VB =
    getViewBinding(
        inflater,
        (javaClass.genericSuperclass as ParameterizedType)
            .actualTypeArguments.filterIsInstance<Class<VB>>()[position],
        parent,
        attachToRoot
    )

@Suppress("UNCHECKED_CAST")
fun <VB : ViewBinding> getViewBinding(
    inflater: LayoutInflater,
    clazz: Class<VB>,
    parent: ViewGroup? = null,
    attachToRoot: Boolean = false
): VB = try {
    clazz.getDeclaredMethod("inflate", LayoutInflater::class.java,ViewGroup::class.java,Boolean::class.java)
        .invoke(null, inflater, parent, attachToRoot) as? VB
} catch (e: Throwable) {
    e.printStackTrace()
    null
} ?: throw RuntimeException("加载ViewBinding失败！")