package com.anjun.core.tool

import android.os.Parcelable
import com.anjun.core.tool.ext.gson
import com.anjun.core.tool.ext.toJson
import com.tencent.mmkv.MMKV
import kotlin.properties.ReadWriteProperty
import kotlin.reflect.KProperty

/**
 *
 * @author melle
 * @since 2024/03/20 15:55:24
 */
@Suppress("UNCHECKED_CAST")
abstract class MMKVReadWrite<T : Any>(private val key: String, private val defaultValue: T) :
    ReadWriteProperty<Any, T> {
    abstract fun getMMKV(): MMKV
    override fun getValue(thisRef: Any, property: KProperty<*>): T {
        return with(getMMKV()) {
            when (defaultValue) {
                is Long -> getLong(key, defaultValue)
                is Float -> getFloat(key, defaultValue)
                is String -> getString(key, defaultValue)
                is Boolean -> getBoolean(key, defaultValue)
                is Int -> getInt(key, defaultValue)
                is Parcelable -> {
                    decodeParcelable(
                        key,
                        defaultValue.javaClass,
                    )?:defaultValue
                }

                else -> {
                    gson.fromJson(
                        getString(key, defaultValue.toJson()),
                        defaultValue::class.java
                    ) ?: defaultValue
                }
            }
        } as T
    }

    override fun setValue(thisRef: Any, property: KProperty<*>, value: T) {
        getMMKV().apply {
            when(value){
                is Long -> putLong(key, value)
                is Float -> putFloat(key, value)
                is String -> putString(key, value)
                is Boolean -> putBoolean(key, value)
                is Int -> putInt(key, value)
                is Parcelable -> encode(key,value)
                else -> {
                    putString(key,value.toJson())
                }
            }
        }
    }

}


