package com.anjun.common.base

import android.annotation.SuppressLint
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.databinding.ViewDataBinding
import androidx.recyclerview.widget.RecyclerView
import androidx.recyclerview.widget.RecyclerView.ViewHolder
import com.anjun.core.mvvm.ext.getViewBinding

/**
 *
 * 封装普通adapter，实现adapter{ViewBinding.bindView()}
 * @author melle
 * @since 2024/03/15 15:28:04
 */
abstract class BaseAdapter<B : ViewDataBinding, T>(
    private var mDataList: MutableList<T> = mutableListOf(),
    private val bindBlock: B.(data: T, position: Int) -> Unit
) : RecyclerView.Adapter<BaseViewHolder<T, B>>() {

    override fun getItemCount(): Int {
        return mDataList.size
    }

    @SuppressLint("NotifyDataSetChanged")
    fun setData(list: MutableList<T>) {
        mDataList = list
        notifyDataSetChanged()
    }

    fun addData(data: T) {
        mDataList.add(data)
        notifyDataSetChanged()
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder<T, B> {
        return BaseViewHolder(
            onCreateBinding(parent),
            bindBlock
        )
    }

    abstract fun onCreateBinding(parent: ViewGroup): B

    override fun onBindViewHolder(holder: BaseViewHolder<T, B>, position: Int) {
        holder.bind(mDataList[position], position)
    }

}

class BaseViewHolder<T, B : ViewDataBinding>(
    private val mBinding: B,
    private val bindBlock: B.(data: T, position: Int) -> Unit
) : ViewHolder(mBinding.root) {
    fun bind(data: T, position: Int) {
        mBinding.bindBlock(data, position)
    }
}

inline fun <reified B : ViewDataBinding, reified T> baseAdapter(
    dataList: MutableList<T> = mutableListOf(),
    noinline bindBlock: B.(data: T, position: Int) -> Unit
) =
    object : BaseAdapter<B, T>(dataList, bindBlock) {
        override fun onCreateBinding(parent: ViewGroup): B {
            return getViewBinding(LayoutInflater.from(parent.context), 0, parent)
        }
    }
