package com.anjun.common.base.activity

import android.view.inputmethod.EditorInfo
import android.widget.TextView
import androidx.databinding.ViewDataBinding
import com.anjun.common.base.databinding.BaseActivityScanViewBinding
import com.anjun.common.view.widget.ClearEditText
import com.anjun.core.mvvm.ext.getViewBinding
import com.anjun.core.tool.ext.hideKeyboard

/**
 *
 * @Description
 * @Author melle
 * @CreateTime 2024/03/12 18:15:54
 */
abstract class BaseScanViewActivity<B: ViewDataBinding>:
    BaseScanActivity<BaseActivityScanViewBinding>() {

    override val mBinding by lazy {
        getViewBinding(layoutInflater, BaseActivityScanViewBinding::class.java).also { it.lifecycleOwner = this }
    }

    protected open val mScanBinding :B by lazy {
        getViewBinding<B>(layoutInflater).also { it.lifecycleOwner = this }
    }

    override fun BaseActivityScanViewBinding.initBindingView() {
        flBaseContent.addView(mScanBinding.root)
        mScanBinding.initScanBinding()
        tvScanTip.setupScanTip()
        cetInput.setupInputTip()
        cetInput.setOnEditorActionListener { tv, actionId, _ ->
            if (EditorInfo.IME_ACTION_DONE == actionId){
                dealCodeResult(tv.text.toString())
                tv.hideKeyboard()
                return@setOnEditorActionListener true
            }
            return@setOnEditorActionListener false
        }
        tvScan.setOnClickListener { startScanCode() }

    }

    override fun onScanResult(result: String) {
        mBinding.cetInput.setText(result)
        dealCodeResult(result)
    }

    abstract fun B.initScanBinding()

    /**
     * 设置输入框上方的提示文本
     */
    abstract fun TextView.setupScanTip()

    /**
     * 设置输入框的提示文本
     */
    abstract fun ClearEditText.setupInputTip()

    /**
     * 处理扫描或者输入数据
     */
    abstract fun dealCodeResult(result: String)
}