package com.anjun.common.base.ext

import com.anjun.common.base.CommonBaseApp
import com.blankj.utilcode.util.TimeUtils
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import java.io.File
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale

/**
 *
 * @Description
 * @Author melle
 * @CreateTime 2024/03/11 14:14:10
 */

val LOG_FILE_DIRECTORY =
    CommonBaseApp.context.externalCacheDir?.path + File.separator + "log" + File.separator
val LOG_FILE_NAME =
    "AjUserLog_${SimpleDateFormat("yyyy_MM_dd_HH_mm_ss", Locale.CHINA).format(Date())}.txt"

/**
 * 写入Log文件，在合适时机将会上传Log文件
 */
fun String.writeToLogFile(
    directory: String = LOG_FILE_DIRECTORY,
    fileName: String = LOG_FILE_NAME
) {
    CoroutineScope(Dispatchers.IO).launch {
        File(directory + fileName).let {
            val builder = StringBuilder()
            if (!it.exists()) {
                it.parentFile?.mkdirs()
                it.createNewFile()
                builder.append("*********以下为安骏物流揽收日志,日志顺序不分先后,请勿删除该文件,否则追究责任.*********\n*********The following is the PDA log of Anjun logistics. The order of the logs is not in any order. Please do not delete or change the files, otherwise the responsibility will be investigated*********\n******** O seguinte é o registo de operações PDA da log ística Anjun. A ordem dos registos não está em nenhuma ordem. Por favor não suprime ou muda os arquivos, caso contrário a responsabilidade será investigada*******\n\n")
            }
            builder.appendLine("**日志：--${this@writeToLogFile}--操作时间：${TimeUtils.getNowString()}（PDA本机时间）")
            it.appendText(builder.toString())
        }
    }
}