package com.anjun.feature.main.home

import android.text.TextUtils
import android.util.Log
import androidx.activity.viewModels
import com.anjun.common.base.activity.AJBaseActivity
import com.anjun.core.tool.ext.getAppVersion
import com.anjun.core.tool.ext.getAppVersionCode
import com.anjun.core.tool.ext.showToast
import com.anjun.core.tool.ext.toJson
import com.anjun.core.tool.utils.DateUtil.YMDFormat
import com.anjun.core.tool.utils.DateUtil.getNowDate
import com.anjun.core.tool.utils.DateUtil.toDate
import com.anjun.data.main.UserSettingMMKV
import com.anjun.data.main.models.VersionModel
import com.anjun.feature.main.R
import com.anjun.feature.main.databinding.ActivityHomeBinding
import com.anjun.feature.main.home.update.UpdateDialog
import com.flyco.tablayout.listener.CustomTabEntity

/**
 *
 * @author melle
 * @since 2024/03/21 10:43:04
 */
class HomeActivity : AJBaseActivity<ActivityHomeBinding>() {

    private val mViewModel: HomeViewModel by viewModels()

    override fun setupData() {
        super.setupData()

        //检查更新
        mViewModel.getVersion().observe(this) { version ->
            version?.let {
                Log.e("melle", it.toJson())
                if (it.compel == 1){
                    //强制更新
                    showUpdate(it)
                }else{
                    if (TextUtils.isEmpty(UserSettingMMKV.noUpdateTime)){
                        showUpdate(it)
                    }else{
                        if (getNowDate(YMDFormat)?.equals(UserSettingMMKV.noUpdateTime.toDate(YMDFormat)) != true){
                            showUpdate(it)
                        }
                    }
                }
            }

        }
    }

    private fun showUpdate(versionModel: VersionModel) {
        if (versionModel.versionCode > getAppVersionCode()){
            if (TextUtils.isEmpty(versionModel.url)){
                return
            }
            UpdateDialog
                .Builder()
                .setNewVersion(getAppVersion())
                .setNowVersion(versionModel.versionInfo)
                .setContent(versionModel.remark)
                .setCancelable(versionModel.compel != 1)
                .setDebug(true)
                .setDownloadUrl(versionModel.url)
                .build()
                .also {
                    it.showUpdateDialog(this)
                    it.setUpdateListener(object :UpdateDialog.OnUpdateListener{
                        override fun onUpdateError() {
                            it.dismissUpdateDialog()
                            getString(R.string.update_error).showToast()
                        }

                        override fun onClose() {
                        }

                    })
                }
        }
    }

    override fun ActivityHomeBinding.initBinding() {
        ctlMenu.setTabData(
            arrayListOf(
                object : CustomTabEntity {
                    override fun getTabTitle() = getString(R.string.s_home)

                    override fun getTabSelectedIcon() = R.drawable.ic_home_fill

                    override fun getTabUnselectedIcon() = R.drawable.ic_home_normal
                },
                object : CustomTabEntity {
                    override fun getTabTitle() = getString(R.string.s_mine)

                    override fun getTabSelectedIcon() = R.drawable.ic_me_fill

                    override fun getTabUnselectedIcon() = R.drawable.ic_me_normal
                },
            ),
            this@HomeActivity,
            R.id.fl_home_content,
            arrayListOf(
                HomeFragment(),
                MineFragment()
            )
        )
    }
}