package com.anjun.feature.main.home.update;
import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.text.Html;
import android.text.TextUtils;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;
import androidx.core.content.FileProvider;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.fragment.app.FragmentTransaction;

import com.anjun.core.tool.utils.DateUtil;
import com.anjun.data.main.UserSettingMMKV;
import com.anjun.feature.main.R;
import com.liulishuo.filedownloader.BaseDownloadTask;
import com.liulishuo.filedownloader.FileDownloadLargeFileListener;
import com.liulishuo.filedownloader.FileDownloader;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;


public class UpdateDialog extends Fragment {

    private ImageView closeIv;
    private TextView versionTv, contentTv, progressTv, fu_now_version, fu_new_version;
    private Button confirmBtn;
    private CheckBox fu_no_tip;
    private String version, content, appUrl, path, nowVersion, newVersion;
    private boolean cancelable;
    //    private OnConfirmListener listener;
    private FragmentActivity mActivity;
    private final int PERMISSION_WRITE_EXTERNAL_STORAGE = 0x100;
    private BaseDownloadTask baseDownloadTask;
    private RelativeLayout progressRl;
    private BarPercentView barPercentView;
    private boolean debug = false;

    public OnUpdateListener mListener;

    public void setUpdateListener(OnUpdateListener listener) {
        mListener = listener;
    }

    public interface OnUpdateListener {
        void onUpdateError();

        void onClose();
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        mActivity = (FragmentActivity) context;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_update, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        FileDownloader.setup(mActivity);
        initViews(view);

    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        if (baseDownloadTask != null && (!baseDownloadTask.isRunning() || baseDownloadTask.pause()))
            baseDownloadTask = null;
    }

    private void initViews(@NonNull View view) {
        closeIv = view.findViewById(R.id.fragment_update_closeIv);
        versionTv = view.findViewById(R.id.fragment_update_versionTv);
        contentTv = view.findViewById(R.id.fragment_update_contentTv);
        fu_no_tip = view.findViewById(R.id.fu_no_tip);
        fu_now_version = view.findViewById(R.id.fu_now_version);
        fu_new_version = view.findViewById(R.id.fu_new_version);
        confirmBtn = view.findViewById(R.id.fragment_update_confirmBtn);
        progressRl = view.findViewById(R.id.fragment_update_progressRl);
        progressTv = view.findViewById(R.id.fragment_update_progressTv);
        barPercentView = view.findViewById(R.id.fragment_update_barPercentView);
        contentTv.setMovementMethod(ScrollingMovementMethod.getInstance());
        closeIv.setVisibility(cancelable ? View.VISIBLE : View.GONE);
        confirmBtn.setOnClickListener(v -> {
//            if (listener != null) {
//                listener.onConfirm(v);
//            } else {
            if (ActivityCompat.checkSelfPermission(mActivity, Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
                requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, PERMISSION_WRITE_EXTERNAL_STORAGE);
            } else {
                downloadApk(appUrl);
            }
//            }
        });
        fu_no_tip.setOnCheckedChangeListener((compoundButton, isNoTip) -> {
            if (isNoTip) {
                // 今日不再弹出
                UserSettingMMKV.INSTANCE.setNoUpdateTime(DateUtil.INSTANCE.getNowDateString(DateUtil.getYMDFormat()));
            } else {
                UserSettingMMKV.INSTANCE.setNoUpdateTime("");
            }
        });

        progressRl.setOnClickListener(v -> {
            if (barPercentView.getProgress() == 1) {
                install(new File(path), mActivity);
            }
        });
        closeIv.setOnClickListener(v -> {
            if (baseDownloadTask == null) {
                dismissUpdateDialog();
                mListener.onClose();
                return;
            }
            if (!baseDownloadTask.isRunning() || baseDownloadTask.pause()) {
                dismissUpdateDialog();
                mListener.onClose();
            }
        });
        versionTv.setText(TextUtils.isEmpty(version) ? "" : version);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            contentTv.setText(Html.fromHtml(TextUtils.isEmpty(content) ? "" : content, Html.FROM_HTML_MODE_LEGACY));
        } else {
            contentTv.setText(Html.fromHtml(TextUtils.isEmpty(content) ? "" : content));
        }
        fu_now_version.setText(nowVersion);
        fu_new_version.setText(newVersion);
    }

    public static class Builder {

        private UpdateDialog updateDialog;

        public Builder() {
            this.updateDialog = new UpdateDialog();
        }

        public Builder setVersion(String version) {
            updateDialog.version = version;
            return this;
        }

        public Builder setNowVersion(String version) {
            updateDialog.nowVersion = version;
            return this;
        }

        public Builder setNewVersion(String version) {
            updateDialog.newVersion = version;
            return this;
        }

        public Builder setContent(String content) {
            updateDialog.content = content;
            return this;
        }

        public Builder setCancelable(boolean cancelable) {
            updateDialog.cancelable = cancelable;
            return this;
        }


        public Builder setDebug(boolean debug) {
            updateDialog.debug = debug;
            return this;
        }

        public Builder setDownloadUrl(String url) {
            updateDialog.appUrl = url;
            return this;
        }

//        public Builder setOnConfirmListener(OnConfirmListener listener) {
//            updateDialog.listener = listener;
//            return this;
//        }

        public UpdateDialog build() {
            return updateDialog;
        }


    }

//    public void setPercent(int progressPercent) {
//        barPercentView.setPercentage(progressPercent);
//        progressTv.setText(progressPercent + "%");
//    }

//    public interface OnConfirmListener {
//        void onConfirm(View view);
//    }

    public void showUpdateDialog(FragmentActivity activity) {
        if (this.isAdded() && this.isVisible()) return;
        FragmentTransaction fragmentTransaction = activity.getSupportFragmentManager().beginTransaction();
        fragmentTransaction.add(android.R.id.content, this);
        fragmentTransaction.setCustomAnimations(R.anim.push_bottom_in, R.anim.push_bottom_out);
        fragmentTransaction.show(this);
        // 下面问题解决方案直接使用commitAllowingStateLoss
        fragmentTransaction.commitAllowingStateLoss();
//        if (activity.isFinishing() || activity.isDestroyed()) {
//            fragmentTransaction.commitAllowingStateLoss();
//        } else {
//            // 如果FragmentTransaction.commit时候activity的状态被保存（即调用了onSaveInstance回调后），就会在这个地方抛这样的异常
//            // java.lang.IllegalStateException: Can not perform this action after onSaveInstanceState
//            // FragmentTransaction.commit接口是不允许在activity的状态保存之后调用（即调用了onSaveInstance回调后）
//            fragmentTransaction.commit();
//        }
    }

    public void dismissUpdateDialog() {
        FragmentTransaction fragmentTransaction = mActivity.getSupportFragmentManager().beginTransaction();
        if (this.isAdded() && this.isVisible()) {
            fragmentTransaction.setCustomAnimations(R.anim.push_bottom_in, R.anim.push_bottom_out);
            fragmentTransaction.hide(this);
            fragmentTransaction.commit();
            new Handler().postDelayed(this::destroyUpdateDialog, 500);
        }
    }

    private void destroyUpdateDialog() {
        FragmentTransaction fragmentTransaction = mActivity.getSupportFragmentManager().beginTransaction();
        fragmentTransaction.hide(this);
        fragmentTransaction.remove(this);
        fragmentTransaction.commit();
    }

    private void downloadApk(String apkUrl) {
        if (TextUtils.isEmpty(apkUrl)) throw new NullPointerException("url can not be empty");
        progressRl.setVisibility(View.VISIBLE);
        confirmBtn.setVisibility(View.GONE);
        // 删除download文件夹下的apk文件
        deleteApk();
        if (apkUrl.endsWith(".apk")) {
            // Log.e(Config.TAG, "------1111111111downloadApkNormal----" + apkUrl);
            downloadApkNormal(apkUrl);
        } else {
            // Log.e(Config.TAG, "------1111111111downloadApkWithNoFileName");
            downloadApkWithNoFileName(apkUrl);
        }
    }

    public void deleteApk() {
        File file = new File("/sdcard/download");
        if (file.exists() && file.isDirectory()) {
            File[] files = file.listFiles();
            if (files != null) {
                if (files.length > 0) {
                    for (File listFile : files) {
                        // 遍历其中的APK文件，进行删除
                        if (listFile.getAbsolutePath().contains(".apk")) {
                            listFile.delete();
                        }
                    }
                }
            }
        }
    }

    private void downloadApkNormal(String apkUrl) {
        String decodeUrl = apkUrl;
        // Log.e(Config.TAG, "------XXXXXXXXXXX  apkUrl = :" + apkUrl);
        try {
            decodeUrl = URLDecoder.decode(apkUrl, "UTF-8");
            // Log.e(Config.TAG, "------222222222URLDecoder.decode");
        } catch (UnsupportedEncodingException e) {
            decodeUrl = apkUrl;
            // Log.e(Config.TAG, "------222222222UnsupportedEncodingException");
        }
        if (debug) Log.e("downloadApk", "originUrl------->" + apkUrl);
        if (debug) Log.e("downloadApk", "decodeUrl------->" + decodeUrl);
        path = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS).getPath() + "/" + decodeUrl.substring(decodeUrl.lastIndexOf("/") + 1);
        // Log.e(Config.TAG, "------3333333333Path:--" + path);
        File dir = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS);
        if (!dir.exists() && dir.mkdir()) {
            startDownloadingApk(decodeUrl);
            // Log.e(Config.TAG, "------444444444 dir is not exists && dir.mkdir()");
        } else {
            startDownloadingApk(decodeUrl);
            // Log.e(Config.TAG, "------444444444 -----------");
        }
    }

    private void startDownloadingApk(String decodeUrl) {
        // Log.e(Config.TAG, "---------------startDownloadingApk:" + decodeUrl + "\n PATH:" + path);
        baseDownloadTask = FileDownloader.getImpl().create(decodeUrl).setPath(path, new File(path).isDirectory()).setCallbackProgressMinInterval(1000).setListener(new FileDownloadLargeFileListener() {

            @Override
            protected void pending(BaseDownloadTask task, long soFarBytes, long totalBytes) {
                // if (debug) Log.e("downloadApk", "pending-------");
                // Log.e(Config.TAG, "------------pending");
            }

            @Override
            protected void progress(BaseDownloadTask task, long soFarBytes, long totalBytes) {
                float percent = 1f * soFarBytes / totalBytes * 100;
                // if (debug) Log.e("downloadApk", "progress-------" + percent);
                if (percent >= 3) {
                    barPercentView.setPercentage(percent);
                    progressTv.setText((int) percent + "%");
                }
            }

            @Override
            protected void paused(BaseDownloadTask task, long soFarBytes, long totalBytes) {
                if (debug) Log.e("downloadApk", "paused-------");
                // Log.e(Config.TAG, "------------paused");
            }

            @Override
            protected void completed(BaseDownloadTask task) {
                if (debug) Log.e("downloadApk", "completed-------");
                barPercentView.setPercentage(100);
                progressTv.setText(100 + "%");
                // TODO install是手动安装入口
                install(new File(path), mActivity);

                // TODO clientInstall是静默安装入口，需root
                // clientInstall(path, mActivity);

                // TODO rootInstallApk是静默安装入口，需root
                // RootsUtils.rootInstallApk(new File(path));
            }

            @Override
            protected void error(BaseDownloadTask task, Throwable e) {
                if (debug) Log.e("downloadApk", "error-------" + e.toString());
                // Log.e(Config.TAG, "---------error:" + e.toString());
            }

            @Override
            protected void warn(BaseDownloadTask task) {
                if (debug) Log.e("downloadApk", "warn-------");
                // Log.e(Config.TAG, "---------------warn");
            }

            @Override
            protected void connected(BaseDownloadTask task, String etag, boolean isContinue, long soFarBytes, long totalBytes) {
                super.connected(task, etag, isContinue, soFarBytes, totalBytes);
                // Log.e(Config.TAG, "---------------connected");
            }

            @Override
            protected void retry(BaseDownloadTask task, Throwable ex, int retryingTimes, long soFarBytes) {
                super.retry(task, ex, retryingTimes, soFarBytes);
                // Log.e(Config.TAG, "---------------retry:" + retryingTimes);
            }
        });
        baseDownloadTask.setAutoRetryTimes(5);   // 设置重试下载次数
        int startTaskId = baseDownloadTask.start();
        // Log.e(Config.TAG, "---------------baseDownloadTask.start()" + startTaskId);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == PERMISSION_WRITE_EXTERNAL_STORAGE) {
            if (grantResults.length > 0
                    && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                // 权限被用户同意，可以去放肆了。
                if (ActivityCompat.checkSelfPermission(mActivity, Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
                    // .e(Config.TAG, "X----------------None PERMISSION");
                } else {
                    downloadApk(appUrl);
                }
            } else {
                // 权限被用户拒绝了，洗洗睡吧。
                // Log.e(Config.TAG, "Y-----------------None PERMISSION");
            }
        }
    }

    /**
     * 安装
     */
    private void install(File filePath, Context context) {
        if (filePath.exists()) {
            startInstall(filePath,context);
        } else {
            // 如果文件不存在！
            downloadApk(appUrl);
        }
    }

    private void startInstall(File filePath,Context context) {
        Intent intent = new Intent(Intent.ACTION_VIEW);
        //判断是否是Android N 以及更高的版本 7.0
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            intent.setFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            Log.d("UpdateDialog", "install: " + filePath);
            Uri contentUri = FileProvider.getUriForFile(context, requireActivity().getApplicationContext().getPackageName() + ".fileProvider", filePath);
            intent.setDataAndType(contentUri, "application/vnd.android.package-archive");
        } else {
            intent.setDataAndType(Uri.fromFile(filePath), "application/vnd.android.package-archive");
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        }
        context.startActivity(intent);
    }

    private void downloadApkWithNoFileName(String appUrl) {
        new Thread() {
            @Override
            public void run() {
                super.run();
                try {
                    URL url = new URL(appUrl);
                    HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
                    urlConnection.setRequestMethod("GET");
                    urlConnection.setConnectTimeout(5000);
                    int responseCode = urlConnection.getResponseCode();
                    if (responseCode == 200) {
                        URL url1 = urlConnection.getURL();
                        downloadApkNormal(url1.toString());
                    } else {
                        com.anjun.common.base.ext.StringExtKt.writeToLogFile("下载APK文件失败，链接请求码不为200!",com.anjun.common.base.ext.StringExtKt.getLOG_FILE_DIRECTORY(),com.anjun.common.base.ext.StringExtKt.getLOG_FILE_NAME());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    com.anjun.common.base.ext.StringExtKt.writeToLogFile("下载APK文件失败，更新出现异常：" + e,com.anjun.common.base.ext.StringExtKt.getLOG_FILE_DIRECTORY(),com.anjun.common.base.ext.StringExtKt.getLOG_FILE_NAME());
                    mListener.onUpdateError();
                }
            }
        }.start();
    }

}